@@ -76,6 +76,10 @@ int main(int argc, char **argv){
     int			pui = 0;		/// progress mode(default text)
     int			raw = 0;
     char		image_hdr_magic[512];
+    int			next=1,next_int=1,next_max_count=7,next_count=7,next_block=0;
+    unsigned long long	next_block_id;
+    char*		cache_buffer;
+    int			nx_current=0;
 
     progress_bar	prog;			/// progress_bar structure defined in progress.h
     image_head		image_hdr;		/// image_head structure defined in partclone.h
@@ -114,6 +118,9 @@ int main(int argc, char **argv){
         m_dialog.percent = 1;
     }
 
+    next_max_count = opt.max_block_cache-1;
+    next_count = opt.max_block_cache-1;
+
     /// print partclone info
     print_partclone_info(opt);
 
@@ -242,12 +249,20 @@ int main(int argc, char **argv){
         if(cmp != 0)
             log_mesg(0, 1, 1, debug, "bitmagic error %i\n", cmp);
 
+	cache_buffer = (char*)malloc(image_hdr.block_size * (next_max_count+1));
+	if(cache_buffer == NULL){
+	    log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+	}
+	buffer = (char*)malloc(image_hdr.block_size); ///alloc a memory to copy data
+	if(buffer == NULL){
+	    log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+	}
+
         /// seek to the first
         sf = lseek(dfw, 0, SEEK_SET);
         log_mesg(1, 0, 0, debug, "seek %lli for writing dtat string\n",sf);
         if (sf == (off_t)-1)
             log_mesg(0, 1, 1, debug, "seek set %lli\n", sf);
-
         /// start restore image file to partition
         for( block_id = 0; block_id < image_hdr.totalblock; block_id++ ){
 
@@ -259,16 +274,7 @@ int main(int argc, char **argv){
                 log_mesg(2, 0, 0, debug, "block_id=%lli, ",block_id);
                 log_mesg(1, 0, 0, debug, "bitmap=%i, ",bitmap[block_id]);
 
-                offset = (off_t)(block_id * image_hdr.block_size);
-#ifdef _FILE_OFFSET_BITS
-                sf = lseek(dfw, offset, SEEK_SET);
-                if (sf == -1)
-                    log_mesg(0, 1, 1, debug, "target seek error = %lli, ",sf);
-#endif
-                buffer = (char*)malloc(image_hdr.block_size); ///alloc a memory to copy data
-                if(buffer == NULL){
-                    log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
-                }
+		memset(buffer, 0, image_hdr.block_size);
                 r_size = read_all(&dfr, buffer, image_hdr.block_size, &opt);
                 log_mesg(1, 0, 0, debug, "bs=%i and r=%i, ",image_hdr.block_size, r_size);
                 if (r_size <0)
@@ -312,17 +318,48 @@ int main(int argc, char **argv){
                     crc_ck2 = crc_ck;
                 }
 
+		if(next != next_count){
+		    memset(cache_buffer, 0, image_hdr.block_size*next_max_count);
+		    for (next_int = 1; next_int <= next_max_count; next_int++)
+		    {
+			next_block_id = block_id+next_int;
+			next_block = bitmap[next_block_id];
+			if (next_block == 1) {
+			    next++;
+			} else if (next_block == 0){
+			    next_count = next;    
+			    break;
+			}
+			next_count = next;
+		    }
+		    log_mesg(1, 0, 0, debug, "next = %i\n",next);
+		}
 
-                /// write block from buffer to partition
-                w_size = write_all(&dfw, buffer, image_hdr.block_size, &opt);
-                log_mesg(1, 0, 0, debug, "bs=%i and w=%i, ",image_hdr.block_size, w_size);
-                if (w_size != (int)image_hdr.block_size)
-                    log_mesg(0, 1, 1, debug, "write error %i \n", w_size);
-                /// free buffer
-                free(buffer);
+		if ((next == next_count) &&(nx_current < next)){
+		    memcpy(cache_buffer+(image_hdr.block_size*nx_current), buffer, image_hdr.block_size);
+		    w_size = 0;
+		    nx_current++;
+		}
+
+		if ((next == next_count) && (nx_current == next)){
+#ifdef _FILE_OFFSET_BITS
+		    offset = (off_t)((block_id-next+1) * image_hdr.block_size);
+		    sf = lseek(dfw, offset, SEEK_SET);
+		    if (sf == -1)
+			log_mesg(0, 1, 1, debug, "target seek error = %lli, ",sf);
+#endif
+		    /// write block from buffer to partition
+		    w_size = write_all(&dfw, cache_buffer, (image_hdr.block_size*nx_current), &opt);
+		    log_mesg(1, 0, 0, debug, "bs=%i and w=%i, ",(image_hdr.block_size*nx_current), w_size);
+		    if (w_size != (int)image_hdr.block_size*nx_current)
+			log_mesg(0, 1, 1, debug, "write error %i \n", w_size);
+		    next = 1;
+		    next_count = next_max_count;
+		    nx_current=0;
+		}
+
+		/// free buffer
                 free(crc_buffer);
-
-
                 copied++;					/// count copied block
                 total_write += (unsigned long long) w_size;	/// count copied size
 
@@ -350,6 +387,9 @@ int main(int argc, char **argv){
 	    if (!opt.quiet)
 		update_pui(&prog, copied, done);
         } // end of for
+	/// free buffer
+	free(cache_buffer);
+	free(buffer);
 	done = 1;
 	update_pui(&prog, copied, done);
         sync_data(dfw, &opt);	
